@@ -36,7 +36,7 @@
 (custom-add-loads 'ConTeXt '("context"))
 (custom-add-loads 'LaTeX-indentation '("latex"))
 (custom-add-loads 'LaTeX '("latex" "tex"))
-(custom-add-loads 'TeX-misc '("plain-tex" "tex"))
+(custom-add-loads 'TeX-misc '("plain-tex" "tex-ispell" "tex"))
 (custom-add-loads 'preview-gs '("preview"))
 (custom-add-loads 'ConTeXt-macro '("context"))
 (custom-add-loads 'TeX-file-extension '("tex"))
