@@ -1,10 +1,12 @@
 ;;; tex-jp.el --- Support for Japanese TeX.  -*- coding: iso-2022-jp-unix; -*-
 
-;; Copyright (C) 1999, 2001-2007, 2012, 2016  Free Software Foundation, Inc.
+;; Copyright (C) 1999, 2001-2008, 2012-2013, 2016-2017
+;;   Free Software Foundation, Inc.
 
 ;; Author:     KOBAYASHI Shinji <koba@flab.fujitsu.co.jp>,
 ;;             Hidenobu Nabetani <nabe@debian.or.jp>
 ;; Maintainer: Masayuki Ataka <masayuki.ataka@gmail.com>
+;;             Ikumi Keita <ikumikeita@jcom.home.ne.jp>
 ;; Keywords: tex
 
 ;; This file is part of AUCTeX.
@@ -111,11 +113,25 @@ For detail, see `TeX-command-list', to which this list is appended."
 				     (const :tag "AmSTeX" ams-tex-mode)))
 			(repeat :tag "Menu elements" :inline t sexp))))
 
-;; Define before first use.
-(defvar japanese-TeX-mode nil
-  "Non-nil means the current buffer handles Japanese TeX/LaTeX.")
-(make-variable-buffer-local 'japanese-TeX-mode)
-(put 'japanese-TeX-mode 'permanent-local t)
+;; customize option $B$N=i4|CM$d(B saved value $B$=$N$b$N$r2~JQ$7$J$$$h$&$K(B
+;; $B$9$k$?$a!"(Bsetcar $B$N;HMQ$OHr$1$k!#(B
+(setq TeX-command-list
+      ;; `TeX-command-list' $B$HF1$89=B$$N?7$7$$(B list $B$r:n$k!#(B
+      ;; $B3FMWAG$N(B list $B$r(B l $B$H$7$F!"(Bl $B$=$N$b$N$r;H$C$?$j!"(Bl $B$r(B
+      ;; $B<c43=$@5$7$?(B list $B$r:n$C$?$j$7$F(B `mapcar' $B$G=8$a$k!#(B
+      (mapcar
+       (lambda (l)
+	 (cond
+	  ;; l $B$NBh(B1$BMWAG$,(B "BibTeX" $B$d(B "Index" $B$@$C$?$i!"(Bl $B$NBh(B2$BMWAG(B
+	  ;; $B$@$1$rF~$lBX$($?JL$N(B list $B$r:n$k!#(B
+	  ((equal (car l) "BibTeX")
+	   (append (list (car l) "%(bibtex) %s") (cddr l)))
+	  ((equal (car l) "Index")
+	   (append (list (car l) "%(makeindex) %s") (cddr l)))
+	  ;; $B$=$l0J30$N>l9g$O(B l $B$=$N$b$N$r;H$&!#(B
+	  (t
+	   l)))
+       TeX-command-list))
 
 ;; $B=gD4$K9T$1$PITMW$K$J$k!#(B
 (setq TeX-command-list
@@ -123,14 +139,15 @@ For detail, see `TeX-command-list', to which this list is appended."
 	      '(("-" "" ignore nil t)) ;; separator for command menu
 	      TeX-command-list))
 
-;; $B;CDj=hCV!#(Btex.el $B$K<h$j9~$s$G$b$i$($k$H$h$$!#(B
-(setcar (cdr (assoc "BibTeX" TeX-command-list)) "%(bibtex) %s")
-(setcar (cdr (assoc "Index" TeX-command-list)) "%(makeindex) %s")
+;; Define before first use.
+(defvar japanese-TeX-mode nil
+  "Non-nil means the current buffer handles Japanese TeX/LaTeX.")
+(make-variable-buffer-local 'japanese-TeX-mode)
+(put 'japanese-TeX-mode 'permanent-local t)
 
-;; $B;CDj=hCV!#(Btex.el $B$K<h$j9~$s$G$b$i$($k$H$h$$!#(B
-(setq TeX-expand-list
+(setq TeX-expand-list-builtin
       (append
-       TeX-expand-list
+       TeX-expand-list-builtin
        '(
         ;; -kanji $B%*%W%7%g%s$NJ8;zNs$r:n$k!#(B
         ("%(kanjiopt)" (lambda ()
@@ -155,10 +172,21 @@ For detail, see `TeX-command-list', to which this list is appended."
                         ((eq TeX-engine 'jtex) "jbibtex")
                         ((eq TeX-engine 'uptex) "upbibtex")
                         (t "bibtex"))))
-        ;; mendex $B$H(B makeindex $B$NE,@Z$JJ}$rA*Br$9$k!#(B
+        ;; mendex, upmendex, makeindex $B$N$&$AE,@Z$J$b$N$rA*Br$9$k!#(B
         ("%(makeindex)" (lambda ()
-                          (if (memq TeX-engine '(ptex uptex))
-                              "mendex %(mendexkopt)" "makeindex")))
+                          (cond
+			   ;; upmendex $B$O(B XeLaTeX $B$d(B LuaLaTeX $B$G$b(B
+			   ;; $B;H$($k!#(Bsee
+			   ;; http://www.t-lab.opal.ne.jp/tex/README_upmendex.md
+			   ;; FIXME: XeLaTeX $B$d(B LuaLaTeX $B$@$1$r;H$&(B
+			   ;; $BMxMQ<T$N>l9g$O!"(Btex-jp $B$O(B load $B$5$l$J(B
+			   ;; $B$$$N$G$3$N@_Dj$O0UL#$,$J$$!#(B
+                           ((and (memq TeX-engine '(uptex xetex luatex))
+                                 (executable-find "upmendex"))
+                            "upmendex %(dic)")
+                           ((memq TeX-engine '(ptex uptex))
+                            "mendex %(mendexkopt) %(dic)")
+                           (t "makeindex"))))
         ;; mendex $BMQF|K\8l%3!<%I%*%W%7%g%s!#(B
         ("%(mendexkopt)" (lambda ()
                            (if (and (featurep 'mule)
@@ -168,6 +196,21 @@ For detail, see `TeX-command-list', to which this list is appended."
                                  (if str (format " -%c " (upcase (aref str 0)))
                                    ""))
                              "")))
+	;; (up)mendex $BMQ<-=q;XDj%*%W%7%g%s!#(B
+	("%(dic)" (lambda ()
+		    ;; master $B$HF1L>$G3HD%;R$,(B .dic $B$N%U%!%$%k$,$"$l$P(B
+		    ;; $B$=$l$r<-=qL>$H$7$F(B -d $B%*%W%7%g%s$KM?$($k!#(B
+		    ;; C-c C-r $BEy$N>l9g(B _region_.dic $B$K$9$Y$-$G$O(B
+		    ;; $B$J$$$N$G!"(B`TeX-master-file' $B$rM[$K8F$V!#(B
+		    (let ((dicname (TeX-master-file "dic" t)))
+		      (if (file-exists-p
+			   (expand-file-name dicname (TeX-master-directory)))
+			  (let ((result (format "-d %s" dicname)))
+			    ;; Advance past the file name in order to
+			    ;; prevent expanding any substring of it.
+			    (setq pos (+ pos (length result)))
+			    result)
+			""))))
         ;; pxdvi $B$H(B %(o?)xdvi $B$NE,@Z$JJ}$rA*Br$9$k!#(B
         ("%(xdvi)" (lambda ()
                      ;; pxdvi $B$O(B ptex, jtex $B6&MQ$J$N$G!"(B
@@ -177,67 +220,57 @@ For detail, see `TeX-command-list', to which this list is appended."
 
 ;;; Viewing (new implementation)
 
-(unless (get 'TeX-view-predicate-list 'saved-value)
-  (setq TeX-view-predicate-list
+(setq TeX-view-predicate-list-builtin
+      (append
        '((paper-a4
-          (TeX-match-style
-           "\\`\\(a4j\\|a4paper\\|a4dutch\\|a4wide\\|sem-a4\\)\\'"))
+	  (let ((regex "\\`\\(?:a4j\\|a4paper\\|a4dutch\\|a4wide\\|sem-a4\\)\\'"))
+	    (or (TeX-match-style regex)
+		(and (fboundp 'LaTeX-match-class-option)
+		     (LaTeX-match-class-option regex)))))
          (paper-a5
-          (TeX-match-style
-           "\\`\\(a5j\\|a5paper\\|a5comb\\)\\'"))
+	  (let ((regex "\\`\\(?:a5j\\|a5paper\\|a5comb\\)\\'"))
+	    (or (TeX-match-style regex)
+		(and (fboundp 'LaTeX-match-class-option)
+		     (LaTeX-match-class-option regex)))))
          ;; jarticle $B$J$I$@$H(B b4paper, b5paper $B$O(B JIS B $B7ONs!#(B
          ;; j-article $B$J$I$NJ}$K$O(B a4j, b5j $B$H$$$C$?%*%W%7%g%s$O$J$$!#(B
          (paper-b5    ; ISO B5
-          (and (TeX-match-style "\\`b5paper\\'")
-               (not (memq TeX-engine '(ptex uptex)))))
+          (and (fboundp 'LaTeX-match-class-option)
+	       (LaTeX-match-class-option "\\`b5paper\\'")
+               (not (TeX-match-style "\\`u?[jt]s?\\(?:article\\|report\\|book\\)\\'"))))
          (paper-b5jis ; JIS B5
-          (or (TeX-match-style "\\`b5j\\'")
-              (and (TeX-match-style "\\`b5paper\\'")
-                   (memq TeX-engine '(ptex uptex)))))
+	  (and (fboundp 'LaTeX-match-class-option)
+	       (or (LaTeX-match-class-option "\\`b5j\\'")
+		   (and (LaTeX-match-class-option "\\`b5paper\\'")
+			(TeX-match-style "\\`u?[jt]s?\\(?:article\\|report\\|book\\)\\'")))))
          ;; article $B$J$I$K$O(B b4paper $B$H$$$&%*%W%7%g%s$O$J$$!#(B
          ;; b4paper $B$H$$$&%*%W%7%g%s$,$"$C$?$i(B JIS B4 $B$H8+$J$9!#(B
          (paper-b4jis
-          (TeX-match-style "\\`\\(b4j\\|b4paper\\)\\'")))))
-;; jsarticle $B$@$HB>$K$b$b$C$HH=7?$N%*%W%7%g%s$,$"$k$,!"(B
-;; $BA4ItLLE]8+$F$k$H%-%j$,$J$$$N$G!"$3$l$/$i$$$G$$$$$@$m$&!#(B
-;; jsarticle.el $B$d(B jsbook.el $B$GDI2CJ,$N=hM}$r;E9~$a$P$$$$$N$+$bCN$l$J$$!#(B
+	  (and (fboundp 'LaTeX-match-class-option)
+	       (LaTeX-match-class-option "\\`\\(?:b4j\\|b4paper\\)\\'"))))
+       ;; jsclasses $B$@$HB>$K$b$b$C$HH=7?$N%*%W%7%g%s$,$"$k$,!"A4ItLLE](B
+       ;; $B8+$F$k$H%-%j$,$J$$$N$G!"$3$l$/$i$$$G$$$$$@$m$&!#(B
+       ;; jsarticle.el $B$d(B jsbook.el $B$GDI2CJ,$N=hM}$r;E9~$a$P$$$$$N$+$bCN$l$J$$!#(B
+       TeX-view-predicate-list-builtin))
 
-;; $B;CDj=hCV!#(Btex.el $B$K<h$j9~$s$G$b$i$($k$H$h$$!#(B
-(unless (get 'TeX-view-program-list 'saved-value)
-  (setq TeX-view-program-list
-       (cond
-        ;; http://oku.edu.mie-u.ac.jp/~okumura/texwiki/?AUCTeX
-        ;; $B$r;29M$K$7$F$_$?!#(B
-        ((eq system-type 'windows-nt)
-         '(("TeXworks" "TeXworks %o" "texworks")
-           ("MuPDF" "mupdf %o" "mupdf")))
-        ;; $B$3$l$G$$$$$N$+$I$&$+$OIT0B!#(B
-        ((eq system-type 'darwin)
-         '(("TeXShop" "open -a TeXShop.app %o" "open")
-           ("TeXworks" "open -a TeXworks.app %o" "open")
-           ("PictPrinter" "open -a PictPrinter.app %d" "open")
-           ("Mxdvi" "open -a Mxdvi.app %d" "open")))
-        (t
-         (setcar (cadr (assoc "xdvi" TeX-view-program-list-builtin))
-                 "%(xdvi) -unique")
-         '(("TeXworks" "texworks %o" "texworks")
-           ("MuPDF" "mupdf %o" "mupdf"))))))
+(unless (memq system-type '(windows-nt darwin))
+  (let ((l (assoc "xdvi" TeX-view-program-list-builtin)))
+    (when l
+      (setcar (cadr l) "%(xdvi) -unique")
+      (setcdr (cdr l) '("%(xdvi)"))))
+  (setq TeX-view-program-list-builtin
+	(append TeX-view-program-list-builtin
+		'(("MuPDF" "mupdf %o" "mupdf")))))
 
 ;; $B$3$l$O(B tex.el $B$K<h$jF~$l$F$b$i$&$N$OFq$7$$$+!)(B
 ;; tex-jp.el $B$,FI$_9~$^$l$k$@$1$G!"(Bdvi viewer $B$N%G%U%)%k%H$,(B dviout $B$K(B
 ;; $B$J$C$F$7$^$&$N$ODq93$,Bg$-$$$+$b!#(B
 (unless (get 'TeX-view-program-selection 'saved-value)
-  (setq TeX-view-program-selection
-       (append
-        (cond
-         ((eq system-type 'windows-nt)
-          '((output-dvi "dviout")
-            (output-pdf "TeXworks")))
-         ((eq system-type 'darwin)
-          '((output-pdf "Preview.app")))
-         (t
-          nil))
-        TeX-view-program-selection)))
+  (if (eq system-type 'windows-nt)
+      (setq TeX-view-program-selection
+	    (append
+	     '((output-dvi "dviout"))
+	     TeX-view-program-selection))))
 
 (mapc (lambda (dir) (add-to-list 'TeX-macro-global dir t))
       (or (TeX-tree-expand
@@ -251,16 +284,6 @@ For detail, see `TeX-command-list', to which this list is appended."
 	   "jlatex" '("/jtex/" "/jbibtex/bst/"))
 	  '("/usr/share/texmf/jtex/" "/usr/share/texmf/jbibtex/bst/")))
 
-;; $B=gD4$K9T$1$PITMW$K$J$k!#(B
-(setq LaTeX-command-style
-      (append '(("\\`u[jt]\\(article\\|report\\|book\\)\\'\\|\\`uplatex\\'"
-                "%(PDF)uplatex %S%(PDFout)")
-               ("\\`[jt]s?\\(article\\|report\\|book\\)\\'"
-                "%(PDF)platex %(kanjiopt)%S%(PDFout)")
-               ("\\`j-\\(article\\|report\\|book\\)\\'"
-                "%(PDF)jlatex %S%(PDFout)"))
-	      LaTeX-command-style))
-
 (defcustom japanese-TeX-error-messages t
   "*If non-nil, explain TeX error messages in Japanese."
   :group 'AUCTeX-jp
@@ -315,13 +338,20 @@ See also a user custom option `TeX-japanese-process-input-coding-system'."
     ;; for upLaTeX
     ("ujarticle") ("ujreport") ("ujbook")
     ("utarticle") ("utreport") ("utbook"))
-  "*List of Japanese document styles."
+  "*List of Japanese document classes."
   :group 'AUCTeX-jp
   :type '(repeat (group (string :format "%v"))))
 
 (setq LaTeX-style-list
       (append japanese-LaTeX-style-list LaTeX-style-list))
 
+;; text$B!A7O$NL@D+BN!&%4%7%C%/BN;XDj%3%^%s%I$O(B jLaTeX $B$K$O$J$$$h$&$G!"(B
+;; (u)pLaTeX $B$G$7$+;H$($J$$$,!"LdBj$K$J$k$3$H$O$J$$$@$m$&!#(B
+(setq LaTeX-font-list
+      (append '((?m "\\textmc{" "}" "\\mathmc{" "}")
+                (?g "\\textgt{" "}" "\\mathgt{" "}"))
+	      LaTeX-font-list))
+
 ;;; Coding system
 
 (defun japanese-TeX-set-process-coding-system (process)
@@ -355,7 +385,7 @@ See also a user custom option `TeX-japanese-process-input-coding-system'."
 		   ((eq TeX-engine 'ptex)
 		    (cond ((eq system-type 'darwin)
 			   'utf-8)
-			  (japanese-TeX-use-kanji-opt-flag
+			  ((and japanese-TeX-use-kanji-opt-flag kanji)
 			   kanji)
 			  (t 'shift_jis)))
 		   ;; jtex $B$J$i(B sjis $B$K8GDj$9$k!#(B
@@ -374,15 +404,38 @@ See also a user custom option `TeX-japanese-process-input-coding-system'."
 		   ;; $B$?$@$7!"(Blocale $B$,F|K\8l$r%5%]!<%H$7$J$$>l9g$O(B
 		   ;; euc $B$K8GDj$9$k!#(B
 		   (t
-		    (if (japanese-TeX-coding-ejsu locale-coding-system)
-			locale-coding-system 'euc-jp))))))
+		    (let ((lcs
+			   (cond
+			    ((boundp 'locale-coding-system)
+			     locale-coding-system)
+			    ;; XEmacs doesn't have `locale-coding-system'.
+			    ;; Instead xemacs 21.5 has
+			    ;; `get-coding-system-from-locale' and
+			    ;; `current-locale'.  They aren't available on
+			    ;; xemacs 21.4.
+			    ((and
+			      (featurep 'xemacs)
+			      (fboundp 'get-coding-system-from-locale))
+			     (get-coding-system-from-locale
+			      (if (fboundp 'current-locale)
+				  (current-locale)
+				;; I don't know XEmacs well, so incorporate
+				;; the suggestion of
+				;; http://lists.gnu.org/archive/html/auctex-devel/2017-02/msg00079.html
+				;; as well.
+				(or (getenv "LC_ALL")
+				    (getenv "LC_CTYPE")
+				    (getenv "LANG")
+				    "")))))))
+		      (if (and lcs (japanese-TeX-coding-ejsu lcs))
+			  lcs 'euc-jp)))))))
 
 	   ;; process $B$KM?$($kF~NO$NJ8;z%3!<%I!#(B
 	   (enc (cond
 		 ;; ptex $B$G(B -kanji $B%*%W%7%g%s$"$j$J$i!"$=$NJ8;z%3!<%I!#(B
 		 ;; $B$J$7$J$i(B utf-8 $B$+(B sjis$B!#(B
 		 ((eq TeX-engine 'ptex)
-		  (if japanese-TeX-use-kanji-opt-flag
+		  (if (and japanese-TeX-use-kanji-opt-flag kanji)
 		      kanji
 		    (if (eq system-type 'windows-nt)
 			'shift_jis 'utf-8)))
@@ -393,13 +446,20 @@ See also a user custom option `TeX-japanese-process-input-coding-system'."
 		 ;; uptex $B$J$i(B utf-8 $B$K8GDj$9$k!#(B
 		 (t
 		  'utf-8))))
+
+	;; $B2~9T%3!<%I$r;XDj!#(B
+	(setq dec (coding-system-change-eol-conversion
+		   dec
+		   (if (eq system-type 'windows-nt) 'dos 'unix))
+	      enc (coding-system-change-eol-conversion
+		   enc
+		   (if (eq system-type 'windows-nt) 'dos 'unix)))
+
 	;; Customize $BCM$,$"$l$P$=$l$rM%@h!#(B
-	(set-process-coding-system process
-				   (or TeX-japanese-process-output-coding-system dec)
-				   (or TeX-japanese-process-input-coding-system enc))))))
-(when (featurep 'mule)
-  (setq TeX-after-start-process-function
-        #'japanese-TeX-set-process-coding-system))
+	(set-process-coding-system
+	 process
+	 (or TeX-japanese-process-output-coding-system dec)
+	 (or TeX-japanese-process-input-coding-system enc))))))
 
 (defun japanese-TeX-coding-ejsu (coding-system)
   "Convert japanese CODING-SYSTEM to mnemonic string.
@@ -421,16 +481,16 @@ Return nil otherwise."
                 (mule-utf-8 . "utf8") ; for emacs 21, 22
                 ;; utf-8-auto $B$d(B utf-8-emacs $B$rF~$l$kI,MW$O$"$k$N$+!)(B
 
-                ;; xemacs $B$G(B jisx0213 $B$O;H$($k$N$+!);H$($k$H$7$F!"(B
-                ;; $B$=$N(B coding system $BL>$O!)(B
+                ;; xemacs 21.5 with mule $B$K$O!"(Bjisx0213 $B$N(B charset $B$O(B
+                ;; $B$"$k$,$=$lMQ$N(B coding system $B$O$J$$!#(B
                 (euc-jis-2004 . "euc")
                 (iso-2022-jp-2004 . "jis")
                 (japanese-shift-jis-2004 . "sjis")
 
-                ;; xemacs $B$K(B cp932 $B7O$N(B coding system $B$,$"$k$N$+(B
-                ;; $B$I$&$+$b$h$/$o$+$i$J$$!#(B
                 (japanese-cp932 . "sjis")
-                (eucjp-ms . "euc"))))))
+                (eucjp-ms . "euc")
+                (windows-932 . "sjis") ; for xemacs 21.5 with mule
+		)))))
 
 (defun japanese-TeX-get-encoding-string ()
   "Return coding option string for Japanese pTeX family.
@@ -491,14 +551,12 @@ Set `japanese-TeX-mode' to t, and enter `TeX-latex-mode'."
   (when japanese-TeX-mode
 ;    (setq TeX-command-default japanese-LaTeX-command-default)
     (TeX-engine-set
-     ;; class file $BL>$KMj$k$N$O@5$7$$$N$+!)(B
-     ;; jLaTeX $B$K$b(B jarticle $B$O0l1~$"$k$7!"(BpLaTeX $B$G$b<+J,$G(B j-article $B$r(B
-     ;; $B%$%s%9%H!<%k$7$F;H$C$F$$$1$J$$K!$O$J$$!#(B
      (cond
-      ((TeX-match-style "\\`u[jt]\\(article\\|report\\|book\\)\\'\\|\\`uplatex\\'")
+      ((TeX-match-style "\\`u[jt]\\(article\\|report\\|book\\)\\'")
        'uptex)
       ((TeX-match-style "\\`[jt]s?\\(article\\|report\\|book\\)\\'")
-       'ptex)
+       (if (LaTeX-match-class-option "\\`uplatex\\'")
+	   'uptex 'ptex))
       ((TeX-match-style "\\`j-\\(article\\|report\\|book\\)\\'")
        'jtex)
       (t japanese-TeX-engine-default)))
@@ -507,6 +565,14 @@ Set `japanese-TeX-mode' to t, and enter `TeX-latex-mode'."
 ;        (if (and (eq TeX-engine 'ptex) (executable-find "pbibtex"))
 ;            "pBibTeX" "jBibTeX"))
 
+    (when (and (fboundp 'font-latex-add-keywords)
+	       (eq TeX-install-font-lock 'font-latex-setup))
+      ;; jLaTeX $B$K$O$J$$%3%^%s%I$@$,!"$=$l$O$b$&5$$K$7$J$/$F$$$$$@$m$&!#(B
+      (font-latex-add-keywords '(("textgt" "{") ("mathgt" "{"))
+			       'bold-command)
+      (font-latex-add-keywords '("gtfamily")
+			       'bold-declaration))
+
     ;; The value of `major-mode' should be `latex-mode', not
     ;; `japanese-latex-mode', because the name `latex-mode' is hard
     ;; coded in several places of AUCTeX like "(eq major-mode
